"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const fs_1 = require("fs");
const log = require("../../utils/log");
const transform_1 = require("../../graph/transform");
const nodes_1 = require("../nodes");
const glob_1 = require("../../utils/glob");
exports.analyseSourcesTransform = transform_1.transformFromPromise((graph) => __awaiter(void 0, void 0, void 0, function* () {
    const packageNode = graph.find(nodes_1.isPackage);
    const entryPoints = graph.filter(x => nodes_1.isEntryPoint(x) && x.state !== 'done');
    const scanner = ts.createScanner(ts.ScriptTarget.Latest, true);
    for (const entryPoint of entryPoints) {
        yield analyseEntryPoint(packageNode, entryPoint, entryPoints, scanner);
    }
    return graph;
}));
/**
 * Analyses an entrypoint, searching for TypeScript dependencies and additional resources (Templates and Stylesheets).
 */
function analyseEntryPoint(packageNode, entryPoint, entryPoints, scanner) {
    return __awaiter(this, void 0, void 0, function* () {
        const { cache, data } = entryPoint;
        const { moduleId, basePath } = data.entryPoint;
        log.debug(`Analysing sources for ${moduleId}`);
        const tsFiles = yield glob_1.globFiles(`${basePath}/**/*.ts`, {
            absolute: true,
            cache: packageNode.cache.globCache,
            ignore: [
                '**/node_modules/**',
                '**/.git/**',
                `${packageNode.data.dest}/**`,
                ...(packageNode.data.primary.moduleId === moduleId
                    ? entryPoints.filter(e => e !== entryPoint).map(e => `${e.data.entryPoint.basePath}/**`)
                    : []),
            ],
        });
        const potentialDependencies = new Set();
        for (const filePath of tsFiles) {
            const entry = cache.sourcesFileCache.getOrCreate(filePath);
            let content;
            if (entry.content === undefined) {
                // file already loaded previously
                content = fs_1.readFileSync(filePath, 'utf8');
                entry.content = content;
                entry.exists = true;
                if (!content) {
                    // if file is blank skip
                    continue;
                }
            }
            else {
                // Previously processed.
                continue;
            }
            scanner.setText(content);
            let token = scanner.scan();
            while (token !== ts.SyntaxKind.EndOfFileToken) {
                if (token === ts.SyntaxKind.ImportKeyword || token === ts.SyntaxKind.ExportKeyword) {
                    while (token !== ts.SyntaxKind.StringLiteral && token !== ts.SyntaxKind.EndOfFileToken) {
                        token = scanner.scan();
                    }
                    const moduleName = scanner.getTokenValue();
                    // Child Entry points need to start with primary entrypoint
                    if (moduleName.startsWith(packageNode.data.primary.moduleId)) {
                        potentialDependencies.add(moduleName);
                    }
                }
                token = scanner.scan();
            }
        }
        const entryPointsMapped = {};
        for (const dep of entryPoints) {
            entryPointsMapped[dep.data.entryPoint.moduleId] = dep;
        }
        for (const moduleName of potentialDependencies) {
            const dep = entryPointsMapped[moduleName];
            if (dep) {
                log.debug(`Found entry point dependency: ${moduleId} -> ${moduleName}`);
                if (moduleId === moduleName) {
                    throw new Error(`Entry point ${moduleName} has a circular dependency on itself.`);
                }
                entryPoint.dependsOn(dep);
            }
            else {
                throw new Error(`Entry point ${moduleName} which is required by ${moduleId} doesn't exists.`);
            }
        }
    });
}
//# sourceMappingURL=analyse-sources.transform.js.map